True or False
False and True
True and False or False and True
(True and False) or (False and True)
True and (False or False) and True
True or False and False or True
(True or False) and (False or True)
True or (False and False) or True
